bl_info = {
    "name": "View rotation for mappers",
    "author": "nemyax",
    "version": (0, 2, 20121116),
    "blender": (2, 6, 4),
    "location": "",
    "description": "Rotate the view as in game map editors",
    "warning": "",
    "wiki_url": "",
    "tracker_url": "",
    "category": "3D View"}

import bpy
import mathutils as mu
import math
from bpy.props import FloatProperty

class RotateViewMapperStyle(bpy.types.Operator):
    """Rotate the view camera about the current POV"""
    bl_idname = "view3d.rotate_view_mapper_style"
    bl_label = "Rotate View (Mapper Style)"
    bl_options = {'GRAB_POINTER', 'BLOCKING'}
    speed = FloatProperty(
        name="Speed",
        description="Rotation speed",
        min=0.1,
        max=100.0,
        default=1.0)

    def modal(self, context, event):
        if event.type == 'MOUSEMOVE':
            new_x = event.mouse_x
            new_y = event.mouse_y
            x_delta = (new_x - self.initial_x) * -self.speed_factor
            y_delta = (new_y - self.initial_y) * -self.speed_factor
            elevation_delta = math.atan2(y_delta, 1.0)
            azimuth_delta = math.atan2(x_delta, 1.0)
            old_view_matrix = context.region_data.view_matrix
            view_pos = old_view_matrix.inverted().translation
            new_view_matrix =\
                mu.Matrix.Rotation(azimuth_delta, 4, 'Z') *\
                old_view_matrix.inverted() *\
                mu.Matrix.Rotation(elevation_delta, 4, 'X').inverted()
            new_view_matrix.translation = view_pos
            context.region_data.view_matrix = new_view_matrix
            self.initial_x = new_x
            self.initial_y = new_y
        elif event.value == 'RELEASE':
            return {'FINISHED'}
        return {'RUNNING_MODAL'}

    def invoke(self, context, event):
        if context.area.type == 'VIEW_3D':
            self.initial_x = event.mouse_x
            self.initial_y = event.mouse_y
            self.speed_factor = self.speed * 0.01
            context.window_manager.modal_handler_add(self)
            return {'RUNNING_MODAL'}
        else:
            return {'CANCELLED'}

def register():
    bpy.utils.register_class(RotateViewMapperStyle)

def unregister():
    bpy.utils.unregister_class(RotateViewMapperStyle)

if __name__ == "__main__":
    register()


